# freeVikings - the "Lost Vikings" clone
# Copyright (C) 2005 Jakub "igneus" Pavlik

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


# sprinter.rb
# igneus 1.2.2005

# Trida na miru pro Erika. Sprinter umi rychle behat a skakat, hlavou
# muze rozbijet zdi.

module FreeVikings

  class Sprinter < Viking

    JUMP_DURATION = 1.75

    def initialize(name, start_position)
      super(name, start_position)
      init_images
      @ability = SprinterAbility.new self
      @state.ability = @ability
      @jump_start = nil
    end

    def space_func_on
      return if not on_ground? and not @jump_start
      @ability.space_on
    end

    def space_func_off
      @ability.space_off
      @jump_start = nil
    end

    def jump
      @state.rise
      @jump_start = Time.now.to_f unless @jump_start
    end

    alias_method :_update, :update

    def update
      _update
      if not @jump_start.nil?
        if (Time.now.to_f - @jump_start) >= JUMP_DURATION
          space_func_off
        end
      end
    end

    private
    def init_images
      i_left = Image.new('erik_left.png')
      i_right = Image.new('erik_right.png')

      @image = ImageBank.new(self)

      @image.add_pair('onground_standing_left', i_left)
      @image.add_pair('onground_standing_right', i_right)
      @image.add_pair('onground_moving_left', i_left)
      @image.add_pair('onground_moving_right', i_right)
      @image.add_pair('falling_standing_right', i_right)
      @image.add_pair('falling_standing_left', i_left)
      @image.add_pair('falling_moving_right', i_right)
      @image.add_pair('falling_moving_left', i_left)
      @image.add_pair('falling_jumping_left', i_left)
      @image.add_pair('falling_jumping_right', i_right)
      @image.add_pair('rising_jumping_left', i_left)
      @image.add_pair('rising_jumping_right', i_right)
      @image.add_pair('onground_jumping_left', i_left)
      @image.add_pair('onground_jumping_right', i_right)

      @portrait = Portrait.new 'erik_face.tga', 'erik_face_unactive.gif', 'dead_face.png'
    end
  end # class
end # module
