# freeVikings - the "Lost Vikings" clone
# Copyright (C) 2005 Jakub "igneus" Pavlik

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


# verticalstate.rb
# igneus 23.5.2005

# Abstraktni trida VerticalState a jeji podtridy.
# Soucast noveho pojeti myslenky VikingState.

=begin
= VerticalState
VerticalState subclasses' objects are used in VikingState to hold
the y-axis movement state information.
=end

module FreeVikings

  class VerticalState

    include StateProprieties

    def initialize(wrapper)
      @wrapper = wrapper
    end

    def velocity
      @velocity.value
    end

    def rise
      @wrapper.vertical_state = RisingState.new @wrapper
    end

    def fall
      @wrapper.vertical_state = FallingState.new @wrapper
    end

    def descend # dosednuti, zastaveni padu.
      @wrapper.vertical_state = OnGroundState.new @wrapper
    end
  end # class VerticalState


  class OnGroundState < VerticalState
    include NotMovingStateProprieties

    def initialize(wrapper)
      super wrapper
      @velocity = Velocity.new 0
    end

    def to_s
      'onground'
    end
  end # class OnGroundState

  class FallingState < VerticalState
    include MovingStateProprieties

    def initialize(wrapper)
      super wrapper
      @velocity = Velocity.new VELOCITY_BASE
    end

    def to_s
      'falling'
    end
  end # class FallingState

  class RisingState < VerticalState
    include MovingStateProprieties

    def initialize(wrapper)
      super wrapper
      @velocity = Velocity.new( - VELOCITY_BASE)
    end

    def to_s
      'rising'
    end
  end # class RisingState
end # module FreeVikings
